/*
 * Copyright (C) 2021 eichehome
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package server;

import java.io.EOFException;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.logging.Level;
import java.util.logging.Logger;
import utils.Message;

/**
 * Diese Klasse stellt einen Thread dar, der auf Nachrichten eines Clients
 * lauscht und diese dann weiter gibt.
 *
 * @version 1.0.0
 * @author eichehome
 */
public class ClientReciveMessageThread extends Thread {

    /**
     * Attribut, welches einen Zeiger auf das zentrale Verzeichnis aller
     * verbundenen Clients enthält.
     */
    private ClientStore clientStore;
    /**
     * Attribut, welches den von Client kommenden Stream repräsentiert.
     */
    private final ObjectInputStream in;

    /**
     * Konstruktor
     *
     * @param in Stream, in dem die Nachrichten eines Clients ankommen.
     * @param clientStore Zeiger auf das zentrale Verzeichnis der Clients, um
     * alle benachrichtigen zu können.
     */
    public ClientReciveMessageThread(ObjectInputStream in, ClientStore clientStore) {
        this.in = in;
        this.clientStore = clientStore;
    }

    /**
     * Hauptmethode des Threads, in der alles abläuft.
     */
    public void run() {
        Logger.getLogger(ClientReciveMessageThread.class.getName()).log(Level.INFO, String.format("Reciveing Thread %d: Gestartet", this.getId()));
        boolean continueLoop = true;
        while (continueLoop) {
            try {
                Message empfangen = null;
                while ((empfangen = (Message) this.in.readObject()) != null) {//Der Thread pleibt hier stehen bis er eine Nachricht vom Client erhält.
                    Logger.getLogger(ClientReciveMessageThread.class.getName()).log(Level.INFO, String.format("Reciveing Thread %d: Time: %s Client: %s Message: %s", this.getId(), empfangen.getTime(), empfangen.getUsername(), empfangen.getMessage()));
                    for (ObjectOutputStream stream : clientStore.getAllOutputStreams()) {
                        stream.writeObject(empfangen);
                        stream.flush();
                        Logger.getLogger(ClientReciveMessageThread.class.getName()).log(Level.INFO, String.format("Reciveing Thread %d: Weitergeleitet", this.getId()));
                    }
                }
            } catch (ClassNotFoundException ex) {
                Logger.getLogger(ClientReciveMessageThread.class.getName()).log(Level.SEVERE, String.format("Reciveing Thread %d: Fehler: %s", this.getId(), ex));
            } catch (EOFException ex) {//Diese Exception zeigt, dass sich die Verbundung zum Client geschlossen hat. Die Aufräumarbeiten übernimmt der Sending Thread diese Clients.
                Logger.getLogger(ClientReciveMessageThread.class.getName()).log(Level.INFO, String.format("Reciveing Thread %d: Socket wurde beendet", this.getId()));
                continueLoop = false;
            } catch (IOException ex) {
                Logger.getLogger(ClientReciveMessageThread.class.getName()).log(Level.SEVERE, String.format("Reciveing Thread %d: Fehler: %s", this.getId(), ex));
                continueLoop = false;
            }
        }
    }

}
